@@ -18,8 +18,8 @@
  */
 
 #include "gnomesettings.h"
-#include "hintprovider.h"
 #include "gsettingshintprovider.h"
+#include "hintprovider.h"
 #include "portalhintprovider.h"
 
 #if QT_VERSION >= 0x060000
@@ -41,8 +41,8 @@
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 #include <QDBusMessage>
-#include <QDBusPendingReply>
 #include <QDBusPendingCallWatcher>
+#include <QDBusPendingReply>
 #include <QDBusReply>
 #include <QDBusServiceWatcher>
 
@@ -103,7 +103,7 @@ GnomeSettings::GnomeSettings(QObject *parent)
         watcher->setConnection(QDBusConnection::sessionBus());
         watcher->setWatchMode(QDBusServiceWatcher::WatchForOwnerChange);
         watcher->addWatchedService(QString::fromLatin1("org.freedesktop.portal.Desktop"));
-        connect(watcher, &QDBusServiceWatcher::serviceOwnerChanged, this, [=] (const QString &service, const QString &oldOwner, const QString &newOwner) {
+        connect(watcher, &QDBusServiceWatcher::serviceOwnerChanged, this, [=](const QString &service, const QString &oldOwner, const QString &newOwner) {
             Q_UNUSED(service)
 
             if (newOwner.isEmpty()) {
@@ -213,15 +213,15 @@ bool GnomeSettings::canUseFileChooserPortal() const
 
 bool GnomeSettings::useGtkThemeDarkVariant() const
 {
-    const QString theme = m_hintProvider->gtkTheme();
-
-    if (m_hintProvider->canRelyOnAppearance()) {
+    QString theme = m_hintProvider->gtkTheme();
+    if (qEnvironmentVariableIsSet("QT_STYLE_OVERRIDE")) {
+        /* If QT_STYLE_OVERRIDE we should rely on it */
+        theme = QString::fromLocal8Bit(qgetenv("QT_STYLE_OVERRIDE"));
+    } else if (m_hintProvider->canRelyOnAppearance()) {
         return m_hintProvider->appearance() == PreferDark;
     }
 
-    return theme.toLower().contains("-dark") ||
-           theme.toLower().endsWith("inverse") ||
-           m_hintProvider->appearance() == PreferDark;
+    return theme.toLower().contains("-dark") || theme.toLower().endsWith("inverse") || m_hintProvider->appearance() == PreferDark;
 }
 
 bool GnomeSettings::useGtkThemeHighContrastVariant() const
@@ -256,14 +256,34 @@ GnomeSettings::TitlebarButtonsPlacement GnomeSettings::titlebarButtonPlacement()
     return m_hintProvider->titlebarButtonPlacement();
 }
 
+static QString colorScheme(bool useHighContrast, bool useDarkVariant)
+{
+    QString colorScheme;
+    if (useHighContrast) {
+        colorScheme = useDarkVariant ? QStringLiteral("AdwaitaHighcontrastInverse") : QStringLiteral("AdwaitaHighcontrast");
+    } else {
+        colorScheme = useDarkVariant ? QStringLiteral("AdwaitaDark") : QStringLiteral("Adwaita");
+    }
+
+    return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("color-schemes/") + colorScheme + QStringLiteral(".colors"));
+}
+
 void GnomeSettings::loadPalette()
 {
     if (useGtkThemeHighContrastVariant()) {
-        m_palette = new QPalette(
-            Adwaita::Colors::palette(useGtkThemeDarkVariant() ? Adwaita::ColorVariant::AdwaitaHighcontrastInverse : Adwaita::ColorVariant::AdwaitaHighcontrast));
+        m_palette = new QPalette(Adwaita::Colors::palette(useGtkThemeDarkVariant() ? Adwaita::ColorVariant::AdwaitaHighcontrastInverse
+                                                                                   : Adwaita::ColorVariant::AdwaitaHighcontrast));
     } else {
         m_palette = new QPalette(Adwaita::Colors::palette(useGtkThemeDarkVariant() ? Adwaita::ColorVariant::AdwaitaDark : Adwaita::ColorVariant::Adwaita));
     }
+
+    const QString colorSchemePath = colorScheme(useGtkThemeHighContrastVariant(), useGtkThemeDarkVariant());
+    if (colorSchemePath.isEmpty()) {
+        qCWarning(QGnomePlatform) << "Could not find color scheme " << colorSchemePath;
+        return;
+    }
+
+    qApp->setProperty("KDE_COLOR_SCHEME_PATH", colorSchemePath);
 }
 
 void GnomeSettings::onCursorBlinkTimeChanged()
@@ -367,7 +387,10 @@ QStringList GnomeSettings::styleNames() const
 
     // 2) Use GTK theme
     if (!gtkTheme.isEmpty()) {
-        const QStringList adwaitaStyles = { QStringLiteral("adwaita"), QStringLiteral("adwaita-dark"), QStringLiteral("highcontrast"), QStringLiteral("highcontrastinverse") };
+        const QStringList adwaitaStyles = {QStringLiteral("adwaita"),
+                                           QStringLiteral("adwaita-dark"),
+                                           QStringLiteral("highcontrast"),
+                                           QStringLiteral("highcontrastinverse")};
         if (adwaitaStyles.contains(gtkTheme.toLower())) {
             QString theme = gtkTheme;
 
@@ -405,8 +428,7 @@ QStringList GnomeSettings::styleNames() const
     }
 
     // 5) Use other styles
-    styleNames << QStringLiteral("fusion")
-               << QStringLiteral("windows");
+    styleNames << QStringLiteral("fusion") << QStringLiteral("windows");
 
     return styleNames;
 }
