@@ -67,6 +67,7 @@
 #define SHADOWS_WIDTH 10
 #define TITLEBAR_HEIGHT 37
 #define WINDOW_BORDER_WIDTH 1
+#define TITLEBAR_SEPARATOR_SIZE 0.5
 
 Q_DECL_IMPORT void qt_blurImage(QPainter *p, QImage &blurImage, qreal radius, bool quality, bool alphaOnly, int transposed = 0);
 
@@ -81,11 +82,11 @@ QGnomePlatformDecoration::QGnomePlatformDecoration()
     option.setWrapMode(QTextOption::NoWrap);
     m_windowTitle.setTextOption(option);
 
-    connect(&GnomeSettings::getInstance(), &GnomeSettings::themeChanged, this, [this] () {
+    connect(&GnomeSettings::getInstance(), &GnomeSettings::themeChanged, this, [this]() {
         loadConfiguration();
         forceRepaint();
     });
-    connect(&GnomeSettings::getInstance(), &GnomeSettings::titlebarChanged, this, [this] () {
+    connect(&GnomeSettings::getInstance(), &GnomeSettings::titlebarChanged, this, [this]() {
         loadConfiguration();
         forceRepaint();
     });
@@ -128,8 +129,8 @@ QRectF QGnomePlatformDecoration::minimizeButtonRect() const
     const bool maximizeEnabled = GnomeSettings::getInstance().titlebarButtons().testFlag(GnomeSettings::getInstance().MaximizeButton);
 
     if (GnomeSettings::getInstance().titlebarButtonPlacement() == GnomeSettings::getInstance().RightPlacement) {
-        return QRectF(windowContentGeometry().width() - BUTTON_WIDTH * (maximizeEnabled ? 3 : 2) - (BUTTON_SPACING * (maximizeEnabled ? 2 : 1))
-                          - BUTTON_MARGINS - margins().right(),
+        return QRectF(windowContentGeometry().width() - BUTTON_WIDTH * (maximizeEnabled ? 3 : 2) - (BUTTON_SPACING * (maximizeEnabled ? 2 : 1)) - BUTTON_MARGINS
+                          - margins().right(),
                       (margins().top() - BUTTON_WIDTH + margins().bottom()) / 2,
                       BUTTON_WIDTH,
                       BUTTON_WIDTH);
@@ -195,7 +196,14 @@ QMargins QGnomePlatformDecoration::margins() const
 
 void QGnomePlatformDecoration::paint(QPaintDevice *device)
 {
+#ifdef DECORATION_SHADOWS_SUPPORT // Qt 6.2.0+ or patched QtWayland
+    const Qt::WindowStates windowStates = waylandWindow()->windowStates();
+    const bool active = windowStates & Qt::WindowActive;
+#else
+    const Qt::WindowStates windowStates = window()->windowStates();
     const bool active = window()->handle()->isActive();
+#endif
+
     const QRect surfaceRect = windowContentGeometry();
     const QColor borderColor = active ? m_borderColor : m_borderInactiveColor;
 
@@ -203,7 +211,7 @@ void QGnomePlatformDecoration::paint(QPaintDevice *device)
     p.setRenderHint(QPainter::Antialiasing);
 
 #ifdef DECORATION_SHADOWS_SUPPORT // Qt 6.2.0+ or patched QtWayland
-    const bool maximized = waylandWindow()->windowStates() & Qt::WindowMaximized;
+    const bool maximized = windowStates & Qt::WindowMaximized;
     const bool tiledLeft = waylandWindow()->toplevelWindowTilingStates() & QWaylandWindow::WindowTiledLeft;
     const bool tiledRight = waylandWindow()->toplevelWindowTilingStates() & QWaylandWindow::WindowTiledRight;
     const bool tiledTop = waylandWindow()->toplevelWindowTilingStates() & QWaylandWindow::WindowTiledTop;
@@ -372,7 +380,7 @@ void QGnomePlatformDecoration::paint(QPaintDevice *device)
     // *                              *
     // ********************************
     QPainterPath borderRect;
-    if (!(window()->windowStates() & Qt::WindowMaximized)) {
+    if (!(windowStates & Qt::WindowMaximized)) {
         borderRect.addRoundedRect(0, 0, surfaceRect.width(), margins().top() + 8, 10, 10);
         p.fillPath(borderRect.simplified(), borderColor);
     }
@@ -390,7 +398,7 @@ void QGnomePlatformDecoration::paint(QPaintDevice *device)
     // *                              *
     // ********************************
     QPainterPath roundedRect;
-    if ((window()->windowStates() & Qt::WindowMaximized)) {
+    if ((windowStates & Qt::WindowMaximized)) {
         roundedRect.addRect(0, 0, surfaceRect.width(), margins().top() + 8);
     } else {
         roundedRect
@@ -414,7 +422,7 @@ void QGnomePlatformDecoration::paint(QPaintDevice *device)
     // *|                            |*
     // *------------------------------*
     // ********************************
-    if (!(window()->windowStates() & Qt::WindowMaximized)) {
+    if (!(windowStates & Qt::WindowMaximized)) {
         QPainterPath borderPath;
         // Left
         borderPath.addRect(0, margins().top(), margins().left(), surfaceRect.height() - margins().top() - WINDOW_BORDER_WIDTH);
@@ -442,7 +450,10 @@ void QGnomePlatformDecoration::paint(QPaintDevice *device)
     // ********************************
     p.save();
     p.setPen(borderColor);
-    p.drawLine(margins().left(), margins().top() - WINDOW_BORDER_WIDTH, surfaceRect.width() - margins().right(), margins().top() - WINDOW_BORDER_WIDTH);
+    p.drawLine(QLineF(margins().left(),
+                      margins().top() - TITLEBAR_SEPARATOR_SIZE,
+                      surfaceRect.width() - margins().right(),
+                      margins().top() - TITLEBAR_SEPARATOR_SIZE));
     p.restore();
 
     // Window title
@@ -499,7 +510,7 @@ void QGnomePlatformDecoration::paint(QPaintDevice *device)
     if (GnomeSettings::getInstance().titlebarButtons().testFlag(GnomeSettings::getInstance().MaximizeButton)) {
         renderButton(&p,
                      maximizeButtonRect(),
-                     (window()->windowStates() & Qt::WindowMaximized) ? Adwaita::ButtonType::ButtonRestore : Adwaita::ButtonType::ButtonMaximize,
+                     (windowStates & Qt::WindowMaximized) ? Adwaita::ButtonType::ButtonRestore : Adwaita::ButtonType::ButtonMaximize,
                      m_maximizeButtonHovered && active,
                      m_clicking == Button::Maximize || m_clicking == Button::Restore);
     }
@@ -604,9 +615,11 @@ bool QGnomePlatformDecoration::handleTouch(QWaylandInputDevice *inputDevice,
     if (handled) {
         if (closeButtonRect().contains(local)) {
             QWindowSystemInterface::handleCloseEvent(window());
-        } else if (GnomeSettings::getInstance().titlebarButtons().testFlag(GnomeSettings::getInstance().MaximizeButton) && maximizeButtonRect().contains(local)) {
+        } else if (GnomeSettings::getInstance().titlebarButtons().testFlag(GnomeSettings::getInstance().MaximizeButton)
+                   && maximizeButtonRect().contains(local)) {
             window()->setWindowStates(window()->windowStates() ^ Qt::WindowMaximized);
-        } else if (GnomeSettings::getInstance().titlebarButtons().testFlag(GnomeSettings::getInstance().MinimizeButton) && minimizeButtonRect().contains(local)) {
+        } else if (GnomeSettings::getInstance().titlebarButtons().testFlag(GnomeSettings::getInstance().MinimizeButton)
+                   && minimizeButtonRect().contains(local)) {
             window()->setWindowState(Qt::WindowMinimized);
         } else if (local.y() <= margins().top()) {
             waylandWindow()->shellSurface()->move(inputDevice);
@@ -620,7 +633,7 @@ bool QGnomePlatformDecoration::handleTouch(QWaylandInputDevice *inputDevice,
 
 QRect QGnomePlatformDecoration::windowContentGeometry() const
 {
-#if QT_VERSION >= 0x060000
+#ifdef DECORATION_SHADOWS_SUPPORT // Qt 6.2.0+ or patched QtWayland
     return waylandWindow()->windowContentGeometry() + margins(ShadowsOnly);
 #else
     return waylandWindow()->windowContentGeometry();
@@ -774,7 +787,13 @@ void QGnomePlatformDecoration::processMouseRight(QWaylandInputDevice *inputDevic
 
 void QGnomePlatformDecoration::renderButton(QPainter *painter, const QRectF &rect, Adwaita::ButtonType button, bool renderFrame, bool sunken)
 {
+#ifdef DECORATION_SHADOWS_SUPPORT // Qt 6.2.0+ or patched QtWayland
+    const Qt::WindowStates windowStates = waylandWindow()->windowStates();
+    const bool active = windowStates & Qt::WindowActive;
+#else
+    const Qt::WindowStates windowStates = window()->windowStates();
     const bool active = window()->handle()->isActive();
+#endif
 
     Adwaita::StyleOptions decorationButtonStyle(painter, QRect());
     decorationButtonStyle.setColor(active ? m_foregroundColor : m_foregroundInactiveColor);
