@@ -27,7 +27,7 @@ QGnomePlatformThemePlugin::QGnomePlatformThemePlugin(QObject *parent)
 QPlatformTheme *QGnomePlatformThemePlugin::create(const QString &key, const QStringList &paramList)
 {
     Q_UNUSED(paramList)
-    if (key == "gnome" || key == "gtk3" || key == "qgnomeplatform")
+    if (key == "gnome" || key == "qgnomeplatform")
         return new QGnomePlatformTheme();
     return nullptr;
 }
